@@ -731,20 +731,26 @@ _EXIT_FN = ct.CFUNCTYPE(None, ct.c_int, ct.c_char_p)
 _xt_exit = _EXIT_FN(_xt_exit)
 
 
-def preserve_globals(fn):
+def set_nfproto(fn):
     def new(*args):
-        obj = args[0]
-        obj._restore_globals()
-        try:
-            ret = fn(*args)
-        except Exception:
-            obj._save_globals()
-            raise
-        obj._save_globals()
-        return ret
+        xtobj = args[0]
+        xtables._xtables_set_nfproto(xtobj.proto)
+        return fn(*args)
     return new
 
 
+_xt_globals = xtables_globals()
+_xt_globals.option_offset = 0
+_xt_globals.program_name = version.__pkgname__.encode()
+_xt_globals.program_version = version.__version__.encode()
+_xt_globals.orig_opts = None
+_xt_globals.opts = None
+_xt_globals.exit_err = _xt_exit
+
+
+_loaded_exts = {}
+
+
 class xtables(object):
     _xtables_init_all = _lib_xtables.xtables_init_all
     _xtables_init_all.restype = ct.c_int
@@ -794,13 +800,6 @@ class xtables(object):
     def _xtinit(self, proto, no_alias_check=False):
         self.proto = proto
         self.no_alias_check = no_alias_check
-        self._xt_globals = xtables_globals()
-        self._xt_globals.option_offset = 0
-        self._xt_globals.program_name = version.__pkgname__.encode()
-        self._xt_globals.program_version = version.__version__.encode()
-        self._xt_globals.orig_opts = None
-        self._xt_globals.opts = None
-        self._xt_globals.exit_err = _xt_exit
 
         thismodule = sys.modules[__name__]
         matchname = "_xtables_match_v%d" % (xtables_version)
@@ -812,61 +811,20 @@ class xtables(object):
             raise XTablesError("unknown xtables version %d" %
                                (xtables_version))
 
-        self._loaded_exts = set()
-
-        # make sure we're initializing with clean state
-        self._xt_params = ct.c_void_p(None).value
-        self._matches = ct.c_void_p(None).value
-        self._pending_matches = ct.c_void_p(None).value
-        self._targets = ct.c_void_p(None).value
-        self._pending_targets = ct.c_void_p(None).value
-
-        rv = xtables._xtables_init_all(ct.pointer(self._xt_globals), proto)
+        rv = xtables._xtables_init_all(ct.pointer(_xt_globals), proto)
         if rv:
             raise XTablesError("xtables_init_all() failed: %d" % (rv))
-        self._save_globals()
 
     def __repr__(self):
         return "XTables for protocol %d" % (self.proto)
 
-    def _save_globals(self):
-        # Save our per-protocol libxtables global variables, and set them to
-        # NULL so that we don't interfere with other protocols.
-        null = ct.c_void_p(None)
-        self._xt_params = xtables._xtables_xt_params.value
-        xtables._xtables_xt_params.value = null.value
-        self._matches = xtables._xtables_matches.value
-        xtables._xtables_matches.value = null.value
-        self._pending_matches = xtables._xtables_pending_matches.value
-        xtables._xtables_pending_matches.value = null.value
-        self._targets = xtables._xtables_targets.value
-        xtables._xtables_targets.value = null.value
-        self._pending_targets = xtables._xtables_pending_targets.value
-        xtables._xtables_pending_targets.value = null.value
-
-    def _restore_globals(self):
-        # Restore per-protocol libxtables global variables saved in
-        # _save_globals().
-        xtables._xtables_set_nfproto(self.proto)
-        xtables._xtables_xt_params.value = self._xt_params
-        xtables._xtables_matches.value = self._matches
-        xtables._xtables_pending_matches.value = self._pending_matches
-        xtables._xtables_targets.value = self._targets
-        xtables._xtables_pending_targets.value = self._pending_targets
-
     def _check_extname(self, name):
         if name in [b"", b"ACCEPT", b"DROP", b"QUEUE", b"RETURN"]:
             name = b"standard"
         return name
 
-    def _loaded(self, name):
-        self._loaded_exts.add(name)
-
-    def _is_loaded(self, name):
-        if name in self._loaded_exts:
-            return True
-        else:
-            return False
+    def _loaded(self, name, ext):
+        _loaded_exts['%s___%s' % (self.proto, name)] = ext
 
     def _get_initfn_from_lib(self, name, lib):
         try:
@@ -901,8 +859,6 @@ class xtables(object):
             raise XTablesError("Unknown protocol %d" % (self.proto))
 
     def _try_register(self, name):
-        if self._is_loaded(name):
-            return
         if isinstance(name, bytes):
             name = name.decode()
         if self._try_extinit(name, _lib_xtables):
@@ -914,11 +870,24 @@ class xtables(object):
             if self._try_extinit(name, lib):
                 return
 
-    @preserve_globals
+    def _get_loaded_ext(self, name):
+        ext = _loaded_exts.get('%s___%s' % (self.proto, name), None)
+        return ext
+
+    @set_nfproto
     def find_match(self, name):
         if isinstance(name, str):
             name = name.encode()
         name = self._check_extname(name)
+
+        ext = self._get_loaded_ext(name)
+        if ext is not None:
+            return ext
+
+        xtables._xtables_matches.value = ct.c_void_p(None).value
+        if xtables._xtables_pending_matches:
+            xtables._xtables_pending_matches.value = ct.c_void_p(None).value
+
         match = xtables._xtables_find_match(name, XTF_TRY_LOAD, None)
         if not match:
             self._try_register(name)
@@ -927,14 +896,23 @@ class xtables(object):
                 return match
 
         m = ct.cast(match, ct.POINTER(self._match_struct))
-        self._loaded(m[0].name)
+        self._loaded(m[0].name, m)
         return m
 
-    @preserve_globals
+    @set_nfproto
     def find_target(self, name):
         if isinstance(name, str):
             name = name.encode()
         name = self._check_extname(name)
+
+        ext = self._get_loaded_ext(name)
+        if ext is not None:
+            return ext
+
+        xtables._xtables_targets.value = ct.c_void_p(None).value
+        if xtables._xtables_pending_targets:
+            xtables._xtables_pending_targets.value = ct.c_void_p(None).value
+
         target = xtables._xtables_find_target(name, XTF_TRY_LOAD)
         if not target:
             self._try_register(name)
@@ -943,10 +921,10 @@ class xtables(object):
                 return target
 
         t = ct.cast(target, ct.POINTER(self._target_struct))
-        self._loaded(t[0].name)
+        self._loaded(t[0].name, t)
         return t
 
-    @preserve_globals
+    @set_nfproto
     def save(self, module, ip, ptr):
         _wrap_save(module.save, ct.cast(ct.pointer(ip), ct.c_void_p), ptr)
 
@@ -972,7 +950,7 @@ class xtables(object):
 
     # Dispatch arguments to the appropriate parse function, based upon the
     # extension's choice of API.
-    @preserve_globals
+    @set_nfproto
     def parse_target(self, argv, invert, t, fw, ptr, x6_parse, x6_options):
         _optarg.value = len(argv) > 1 and argv[1] or None
         _optind.value = len(argv) - 1
@@ -1017,7 +995,7 @@ class xtables(object):
 
     # Dispatch arguments to the appropriate parse function, based upon the
     # extension's choice of API.
-    @preserve_globals
+    @set_nfproto
     def parse_match(self, argv, invert, m, fw, ptr, x6_parse, x6_options):
         _optarg.value = len(argv) > 1 and argv[1] or None
         _optind.value = len(argv) - 1
@@ -1098,7 +1076,7 @@ class xtables(object):
 
     # Dispatch arguments to the appropriate final_check function, based upon
     # the extension's choice of API.
-    @preserve_globals
+    @set_nfproto
     def final_check_target(self, target):
         x6_fcheck = None
         try:
@@ -1138,7 +1116,7 @@ class xtables(object):
 
     # Dispatch arguments to the appropriate final_check function, based upon
     # the extension's choice of API.
-    @preserve_globals
+    @set_nfproto
     def final_check_match(self, match):
         x6_fcheck = None
         try:
